#!python3

# PyUPB.py: A Python module to control a Pegasus Ultimate Power Box v2 from the command line.

# Author:	Hans Kostense
# Date:		02-May-20

# Usage w/ py launcher: py PyUPB.py <commandfile>
# Usage: python .\PyUPB.py <commandfile>
# Working Directory: location of PyUPB.py

# PyUPB connects to a Pegasus UPBv2 and sends the series of commands specified in the commandfile on the command line.
# NB: location of command files and of logfiles needs to be configured in separate file config.py.
# Examples of specific commandfiles are:
# - connect-all
# - disconnect-all
# - focus-position-zero
# - focus-session-start

# NB: PyUPB will only be able to connect to the UPB if the Pegasus UPB software is running and connected to the UPB !!!

MODULE = "PyUPB.py: A Python module to control a Pegasus Ultimate Power Box v2"
VERSION = "1.0"
# - - -   I m p o r t s
from sys import argv
from datetime import datetime
from pathlib import Path
import config
import os
import socket
import sys

# - - -   I N I T
# Verify commandfile has been specified or fail gracefully
if len(sys.argv) > 1:
	commandfile = sys.argv[1]
else:
	print('Usage: PyUPB.py <commandfile>')
	sys.exit(1)

UDP_port = config.UDP_port
UDP_IP = config.UDP_IP

# Commandfile folder (input)
dircmd = config.dircmd
commandfile_full = Path(dircmd).joinpath(commandfile)

# Logfile folder (output)
dirlog = config.dirlog
current_time = datetime.now()
logfile = 'PyUPB_logfile_' + current_time.strftime('%Y%m%d_%H%M%S') + '.log'
logfile_full = Path(dirlog).joinpath(logfile)

# - - -   F u n c t i o n s
def send_command(upb_command):
	# Create UDP socket or fail gracefully
	# Timeout is also set to prevent eternal waiting on a reply that never comes
	try:
		upb = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		upb.settimeout(3)
		upb.sendto(upb_command.encode(), (UDP_IP, UDP_port))
		data, addr = upb.recvfrom(1024)
		upb_reply = data.decode()
	except OSError as err:
		lf.write('Unable to communicate with port: {} on '.format(UDP_port)+UDP_IP+'\n')
		lf.write('Ensure Pegasus UPB software is running and connected to UPBv2.\n')
		lf.write('OS error: {0}'.format(err))
		upb_reply = '<no-reply>'
	else:
		upb.close()

	return(upb_reply)

# - - -   M A I N
# Read commandfile and process line by line to send commands to Pegasus UPBv2.
# Executed commands and received replies are written to a logfile for later review.

# Verify logfile can be opened or fail gracefully
try:
    lf = open(logfile_full, 'w')
except OSError as err:
	print('Unable to open: ', logfile)
	print('OS error: {0}'.format(err))
	sys.exit(1)

# Verify commandfile can be opened or fail gracefully
try:
	cf = open(commandfile_full, 'r')
	# Read first command in commandfile
	upb_command = cf.readline()
	# Write commandfile name to logfile
	lf.write('Processing UPB commandfile: '+commandfile+'\n')
	while upb_command:
		# Skip comment lines in commandfile
		if len(upb_command) > 1 and upb_command[:1] != '#':
			# Strip new line character from command (otherwise not accepted by UPBv2)
			upb_command = upb_command.rstrip('\n')
			# Write command to logfile
			lf.write('Command: '+upb_command+'\n')
			# Send command and receive reply from UPBv2
			upb_reply = send_command(upb_command)
			# Write reply to logfile
			lf.write('Reply: '+upb_reply+'\n')
		# Read next command until EOF
		upb_command = cf.readline()
except OSError as err:
	print('Unable to open:', commandfile)
	print('OS error: {0}'.format(err))
	lf.write('Unable to open: '+commandfile+'\n')
	lf.write('OS error: {0}'.format(err))
	lf.close()
	sys.exit(1)
else:
	print('Finished processing UPB commandfile: ', commandfile)
	print('Results are logged in:', logfile)
	lf.write('Finished processing UPB commandfile.\n')
	cf.close()
	lf.close()

